import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
var _excluded = ["onBlur", "isOpen", "onToggle", "children", "errorState", "state", "disabled", "keysSelection", "multiselection", "hasSearch", "searchIcon", "searchPlaceholder", "noResults", "refMoleculeSelect", "aria-label", "onSearch", "readOnly", "onChange", "selectSize"];
import { Children, cloneElement, createRef, forwardRef, useCallback, useEffect, useMemo, useReducer, useRef, useState } from 'react';
import PropTypes from 'prop-types';
import { getTarget } from '@s-ui/js/lib/react';
import { inputSizes as SELECT_SIZES } from '@s-ui/react-atom-input';
import useMergeRefs from '@s-ui/react-hooks/lib/useMergeRefs';
import { moleculeDropdownListSizes as SIZES } from '@s-ui/react-molecule-dropdown-list';
import MoleculeSelectMultipleSelection from './components/MultipleSelection.js';
import MoleculeSelectSingleSelection from './components/SingleSelection.js';
import { DropdownContext, ENABLED_KEYS, getClassName, getOptionData, SELECT_STATES, SELECTION_KEYS } from './config.js';
import { jsx as _jsx } from "react/jsx-runtime";
var useFunctionalRef = function useFunctionalRef() {
  return useReducer(function (_s, node) {
    return node;
  }, null);
};
var MoleculeSelect = /*#__PURE__*/forwardRef(function (_ref, forwardedRef) {
  var onBlur = _ref.onBlur,
    isOpen = _ref.isOpen,
    onToggle = _ref.onToggle,
    children = _ref.children,
    errorState = _ref.errorState,
    state = _ref.state,
    _ref$disabled = _ref.disabled,
    disabled = _ref$disabled === void 0 ? false : _ref$disabled,
    _ref$keysSelection = _ref.keysSelection,
    keysSelection = _ref$keysSelection === void 0 ? SELECTION_KEYS : _ref$keysSelection,
    _ref$multiselection = _ref.multiselection,
    multiselection = _ref$multiselection === void 0 ? false : _ref$multiselection,
    _ref$hasSearch = _ref.hasSearch,
    hasSearch = _ref$hasSearch === void 0 ? false : _ref$hasSearch,
    searchIcon = _ref.searchIcon,
    searchPlaceholder = _ref.searchPlaceholder,
    noResults = _ref.noResults,
    refMoleculeSelectFromProps = _ref.refMoleculeSelect,
    ariaLabel = _ref['aria-label'],
    onSearch = _ref.onSearch,
    _ref$readOnly = _ref.readOnly,
    readOnly = _ref$readOnly === void 0 ? false : _ref$readOnly,
    _ref$onChange = _ref.onChange,
    onChange = _ref$onChange === void 0 ? function () {} : _ref$onChange,
    _ref$selectSize = _ref.selectSize,
    selectSize = _ref$selectSize === void 0 ? SELECT_SIZES.MEDIUM : _ref$selectSize,
    props = _objectWithoutPropertiesLoose(_ref, _excluded);
  var refOptions = useRef({});
  var refMoleculeSelect = useRef(refMoleculeSelectFromProps);
  var refsMoleculeSelectOptions = useRef([]);
  var ref = useMergeRefs(forwardedRef, refMoleculeSelect);
  var _useFunctionalRef = useFunctionalRef(),
    inputSearch = _useFunctionalRef[0],
    setInputRef = _useFunctionalRef[1];
  var _useState = useState(isOpen || false),
    isOpenState = _useState[0],
    setIsOpenState = _useState[1];
  useEffect(function () {
    return setIsOpenState(isOpen);
  }, [isOpen, setIsOpenState]);
  Object.assign(refOptions.current, getOptionData(children));
  var _useState2 = useState(false),
    focus = _useState2[0],
    setFocus = _useState2[1];
  var extendedChildren = useMemo(function () {
    return Children.toArray(children).filter(Boolean).map(function (child, index) {
      refsMoleculeSelectOptions.current[index] = /*#__PURE__*/createRef();
      return /*#__PURE__*/cloneElement(child, {
        ref: refsMoleculeSelectOptions.current[index],
        selectKey: keysSelection
      });
    });
  }, [children, keysSelection]);
  var numOptions = Children.toArray(extendedChildren).length;
  var className = getClassName({
    state: state,
    errorState: errorState,
    focus: focus,
    disabled: disabled
  });
  var handleToggle = useCallback(function (ev, _temp) {
    var _ref2 = _temp === void 0 ? {
        isOutsideEvent: false
      } : _temp,
      isOpen = _ref2.isOpen,
      isOutsideEvent = _ref2.isOutsideEvent;
    setIsOpenState(isOpen !== undefined ? isOpen : !isOpenState);
    typeof onToggle === 'function' && onToggle(ev, {
      isOpen: isOpen
    });
    if (!isOutsideEvent) {
      ev.preventDefault();
      ev.stopPropagation();
    }
  }, [isOpenState, onToggle]);
  var closeList = useCallback(function (ev, _ref3) {
    var _ref3$isOutsideEvent = _ref3.isOutsideEvent,
      isOutsideEvent = _ref3$isOutsideEvent === void 0 ? false : _ref3$isOutsideEvent;
    handleToggle(ev, {
      isOpen: false,
      isOutsideEvent: isOutsideEvent
    });
  }, [handleToggle]);
  var handleOutsideClick = useCallback(function (ev) {
    if (disabled) return;
    if (refMoleculeSelect.current && !refMoleculeSelect.current.contains(ev.target)) {
      // outside click
      closeList(ev, {
        isOutsideEvent: true
      });
      setFocus(false);
    }
  }, [closeList, disabled]);
  var focusFirstOption = useCallback(function (ev) {
    var options = refsMoleculeSelectOptions.current.map(function (option) {
      return getTarget(option.current);
    });
    options[0] && options[0].focus();
    ev.preventDefault();
    ev.stopPropagation();
  }, [refsMoleculeSelectOptions]);
  var focusSearchInput = useCallback(function (ev) {
    var isEnabledKey = ENABLED_KEYS.includes(ev == null ? void 0 : ev.key);
    isEnabledKey ? focusFirstOption(ev) : inputSearch == null ? void 0 : inputSearch.focus();
    ev == null ? void 0 : ev.preventDefault();
    ev == null ? void 0 : ev.stopPropagation();
  }, [inputSearch, focusFirstOption]);
  var isFirstOptionFocused = useCallback(function () {
    var options = refsMoleculeSelectOptions.current.map(function (option) {
      return getTarget(option.current);
    });
    return document.activeElement === options[0];
  }, [refsMoleculeSelectOptions]);
  useEffect(function () {
    Object.assign(refOptions.current, getOptionData(children));
    document.addEventListener('touchend', handleOutsideClick);
    document.addEventListener('mousedown', handleOutsideClick);
    return function () {
      document.removeEventListener('touchend', handleOutsideClick);
      document.removeEventListener('mousedown', handleOutsideClick);
    };
  }, [children, handleOutsideClick]);
  useEffect(function () {
    isOpenState && setTimeout(function () {
      return focusSearchInput();
    });
  }, [isOpenState, focusSearchInput]);
  var handleKeyDown = function handleKeyDown(ev) {
    ev.persist();
    var isEnabledKey = ENABLED_KEYS.includes(ev.key);
    if (!isOpenState && isEnabledKey) {
      handleToggle(ev, {
        isOpen: !isOpenState
      });
      setTimeout(function () {
        return focusFirstOption(ev);
      });
    } else if (ev.key === 'Escape') {
      closeList(ev, {
        isOutsideEvent: true
      });
    } else {
      isOpenState && setTimeout(function () {
        return focusSearchInput(ev);
      });
      !hasSearch && setTimeout(function () {
        return focusFirstOption(ev);
      });
    }
  };
  var handleFocusOut = function handleFocusOut(event) {
    typeof onBlur === 'function' && onBlur(event);
    setFocus(false);
  };
  var handleFocusIn = function handleFocusIn() {
    return !disabled && !hasSearch && setFocus(true);
  };
  var handleClick = function handleClick(ev) {
    ev.persist();
    if (focus) {
      !hasSearch && setTimeout(function () {
        return focusFirstOption(ev);
      });
    }
  };
  var Select = multiselection ? MoleculeSelectMultipleSelection : MoleculeSelectSingleSelection;
  var context = {
    hasSearch: hasSearch,
    setInputRef: setInputRef,
    isOpen: isOpenState,
    inputSearch: inputSearch,
    onSearch: onSearch,
    isFirstOptionFocused: isFirstOptionFocused,
    searchPlaceholder: searchPlaceholder,
    searchIcon: searchIcon
  };
  return /*#__PURE__*/_jsx(DropdownContext.Provider, {
    value: context,
    children: /*#__PURE__*/_jsx("div", {
      ref: ref,
      tabIndex: "0",
      className: className,
      onKeyDown: handleKeyDown,
      onFocus: handleFocusIn,
      onBlur: handleFocusOut,
      onClick: handleClick,
      "aria-label": ariaLabel,
      children: /*#__PURE__*/_jsx(Select, _extends({
        refMoleculeSelect: refMoleculeSelect,
        optionsData: refOptions.current,
        isOpen: isOpenState
      }, props, {
        state: state,
        disabled: disabled,
        readOnly: readOnly,
        onToggle: handleToggle,
        onChange: onChange,
        size: selectSize,
        multiselection: multiselection,
        keysSelection: keysSelection,
        children: numOptions > 0 ? extendedChildren : noResults
      }))
    })
  });
});
MoleculeSelect.displayName = 'MoleculeSelect';
export default MoleculeSelect;
export { SIZES as moleculeSelectDropdownListSizes };
export { SELECT_SIZES as moleculeSelectSizes };
export { SELECT_STATES as moleculeSelectStates };